
from .bidding_zones import *
from .countries import *
from .markets import *
from .Price_Logic import *
from .timeframes import *
from .customBaseModel import *

__all__ = [name for name in dir() if not name.startswith('_')] #type: ignore


from spotON_sdk import all_Countries
from pprint import pprint

pprint (all_Countries.return_List)
pprint (all_Countries.get_country_by_name("Sweden"))

country_code = "AT"
if "_" in country_code:
    cleaned_code = country_code.split("_")[0]
else:
    cleaned_code = country_code
print(cleaned_code)



from pprint import pprint as pprint

from spotON_sdk import Price_Logic,Markets,Market,Timeframes,Continuous_On_Time,Timeframe
from spotON_sdk.Logic.Price.Price_Logic import Interrupted_On_Time

import base64
#  Test 1 Create Price_Logic Objects
timeframes = Timeframes()
timeframes.add_timeframe(1,5)
price_Logic1 = Price_Logic(nr_of_hours_on=10,market=Markets.germany,timeframes=timeframes,pricefinding=Continuous_On_Time(week=1,best_hour=1))
price_Logic2 = Price_Logic(nr_of_hours_on=10,market=Markets.germany,timeframes=timeframes,pricefinding=Interrupted_On_Time())
price_Logic3 = Price_Logic(nr_of_hours_on=24,market=Markets.germany)
timeframes = Timeframe(start=1,end=5)
print (f"{price_Logic1.timeframes.possible_hours=}")
pprint(price_Logic3.dict())

# Test 2 JSON 
price_Logic1_json = price_Logic1.to_json()

print(price_Logic1_json)

# Test 3 create API_Call object
from spotON_sdk import API_Call
from spotON_helper.authentification.expireing_Token import generate_token,verify_token
# wirte a test token
token = generate_token('test_user_id')
print(f"{token=}")

# verify the token
result = verify_token(token)
print(f"{result=}")

apiCall = API_Call(spotOn_API_code=token,price_logic=price_Logic1)

api_call_data_Json = apiCall.as_json(escaped=False)
print(f"{api_call_data_Json=}")

api_call_instance = API_Call.parse_raw(api_call_data_Json)
print(f"{api_call_instance=}")
print(f"{type(api_call_instance)=}")


escaped_api_call_data_Json = apiCall.as_json(escaped=True)
print(f"{escaped_api_call_data_Json=}")




from dataclasses import dataclass
from ..Logic.Price.customBaseModel import CustomBaseModel
from ..Logic.Price.Price_Logic import Price_Logic
import json

class API_Call(CustomBaseModel):
    spotOn_API_code: str
    price_logic : Price_Logic

    def as_json(self,escaped=False):

        API_Call_dict = self.to_dict()
        json_str = json.dumps(API_Call_dict)
        if escaped:
            json_str = json.dumps(json_str)

        return json_str
    



import aiohttp
import json

from spotON_sdk import Price_Logic,Continuous_On_Time,Markets,Timeframes

async def request_Day_Ahead_Price(data:API_Call):
           
    url = 'https://km28t5snp5.execute-api.eu-central-1.amazonaws.com/Prod/echo'
    #url = 'http://127.0.0.1:8000/echo'



    headers = {'Content-type': 'application/json'} # Set the content type to JSON

    async with aiohttp.ClientSession() as session:
        async with session.post(url, data=data.json(), headers=headers) as resp:
            print("Header:",resp.headers)
            print("Status:", resp.status)
            print("Content-type:", resp.headers['content-type'])
            #print(data)
            response = await resp.json()
            
            print(response)







import pandas as pd

def preprocess(df):
    df.index =  pd.to_datetime(df.index, utc=False)
    df = df.rename_axis('DateTime')
    df = df.rename(columns={"value":"Price"})
    df['Week'] = df.index.isocalendar().week
    df['Day'] = df.index.day
    df["Hour"] = df.index.hour

    # Calculate the daily average price and store it in a new DataFrame
    daily_avg = df.resample('D')['Price'].mean()

    # Merge the two DataFrames using the DateTime index with a left join
    merged_df = pd.merge(df, daily_avg, left_index=True, right_index=True, how='left', suffixes=('', '_daily_AVG'))

    # Forward fill the missing values in the 'EXXA Price_daily' column
    merged_df['Price_daily_AVG'].fillna(method='ffill', inplace=True)
    return merged_df

def add_priceRating(df):
    df['rating'] = df['Price'] - df['Price_daily_AVG']
    df['rating_rank'] = df.groupby(df.index.date)['rating'].rank(ascending=True)-1
    return df

def add_priceRatings(df):
    df['savings'] = df['Price'] - df['Price_daily_AVG']
    df['rating_rank'] = df.groupby(df.index.date)['savings'].rank(ascending=True)-1
    return df

def add_timeslot_savings(df):
    df['savings'] = df['Price'] - df['Price_daily_AVG']
    for timeslot_length in range(1,24):
        df[f"timeslot_{timeslot_length}H"] = df["savings"].rolling(timeslot_length).mean().shift(-timeslot_length+1)
    return df


def reduce_Filesize(df:pd.DataFrame):
    df["Price"] = df["Price"].astype('float16')
    df["Week"] = df["Week"].astype("int8")
    df["Day"] = df["Day"].astype("int8")
    df["Hour"] = df["Hour"].astype("int8")
    df["Price_daily_AVG"] = df["Price_daily_AVG"].astype('float16')
    df["savings"] = df["savings"].astype('float16')
    for timeslot_length in range(1,24):
        df[f"timeslot_{timeslot_length}H"] = df[f"timeslot_{timeslot_length}H"].astype('float16')
    return df




from typing import Any, List, Union
from pydantic import  Field, validator, root_validator
import json

from .timeframes import Timeframes,Timeframe, hours_to_timeframes
from .markets import Market, Markets
from .customBaseModel import CustomBaseModel
from pprint import pprint

class PricefindingError(ValueError):
    pass

class Interrupted_On_Time(CustomBaseModel):
    name: str = Field(default="Interrupted_On_Time",init = False)
    minimum_hours_on: int = Field(default=1,init = False)
    pass

class Continuous_On_Time(CustomBaseModel):
    name: str = Field(default="Continuous_On_Time",init = False)
    week: int = Field(default=None)
    best_hour: int = Field(default=None)

class Price_Logic(CustomBaseModel):
    nr_of_hours_on: int
    market: Market   
    timeframes: Timeframes = Timeframes()
    pricefinding: Union[Continuous_On_Time, Interrupted_On_Time] = Interrupted_On_Time()
    resolution: float = Field(default=1)
    timeframe_shorter_than_nr_of_hours_on: bool = Field(default=False)

    def __init__(self, **data):
        super().__init__(**data)
        self.market: Market  # This annotation tells the linter that `market` is of type `Market`

        if data.get("pricefinding") is not None:
            self.pricefinding = data["pricefinding"]

    def update_pricefinding(self, pricefinding: Union[Continuous_On_Time, Interrupted_On_Time]):
        if not isinstance(pricefinding, (Continuous_On_Time, Interrupted_On_Time)):
            raise ValueError("Invalid type for pricefinding")
        self.pricefinding = pricefinding

    @root_validator(pre=True)
    def set_market_to_market_object(cls, values):
        if isinstance(values["market"], str):
            result = Markets.get_market_by_name(values["market"])
            result = Markets.get_market_by_code(values["market"])
            if result is None:
                raise ValueError("Market is not valid")
            values["market"] = result
        return values


    def update_timeframes(self, timeframes: Timeframes):
        self.timeframes = timeframes
    


    def to_json(self) -> str:
        switch_pattern_dict = self.to_dict()
        json_str = json.dumps(switch_pattern_dict)

        return json_str
    
    def pack(self) -> dict[str, Any]:
        '''Pack the object into a dictionary'''
        details = {
            "area": self.market.area.name,  
            "nr_of_hours_on": self.nr_of_hours_on,
            "timeframes": self.timeframes.possible_hours,
            "pricefinding": self.pricefinding.name,
            "minimum_hours_on": self.pricefinding.minimum_hours_on#type: ignore
        }
        return details


    @staticmethod
    def unpack(details: dict[str, Any]) -> "Price_Logic":
        pprint (f"{details=}")
        '''Unpack the dictionary into a Price_Logic object'''
        market = Markets.get_market_by_code(details["area"])
        nr_of_hours_on = details["nr_of_hours_on"]
        timeframes_decimal = details['timeframes']
        minimum_hours_on = details['minimum_hours_on']
        # Convert 'timeframes' list of Decimal to list of int
        timeframes_int = [int(tf) for tf in timeframes_decimal]
        print(f"timeframes_int before unpacking: {timeframes_int}")
        # If there's a function called 'hours_to_timeframes' you want to use, apply it here
        timeframes = hours_to_timeframes(timeframes_int)
        print(f"timeframes after unpacking: {timeframes}")
        pricefinding_class_string = details["pricefinding"]

        price_logic = Price_Logic(market=market, nr_of_hours_on=nr_of_hours_on)
        price_logic.update_timeframes(timeframes)


        if pricefinding_class_string == Interrupted_On_Time().name:
            price_logic.update_pricefinding(Interrupted_On_Time())
            price_logic.pricefinding.minimum_hours_on = int(minimum_hours_on)
        elif pricefinding_class_string == Continuous_On_Time().name:
            price_logic.update_pricefinding(Continuous_On_Time())
        else:
            raise PricefindingError
        

        

        print (f"{price_logic=}")

        return price_logic



    @validator('nr_of_hours_on')
    def validate_nr_of_hours_on(cls, v):
        if v <= 0:
            raise ValueError('nr_of_hours_on must be greater than 0')
        return v

    @root_validator
    def validate_timeframes(cls, values):
        timeframes = values.get('timeframes')
        nr_of_hours_on = values.get('nr_of_hours_on')
        if timeframes and nr_of_hours_on:
            possible_hours = timeframes.possible_hours
            if nr_of_hours_on > len(possible_hours):
                values['timeframe_shorter_than_nr_of_hours_on'] = True
            else:
                values['timeframe_shorter_than_nr_of_hours_on'] = False
        return values


from entsoe import Area 
from dataclasses import dataclass
from enum import Enum

@dataclass
class Area_details:
    name: str
    code: str
    tz: str

@dataclass
class spotON_Area():
    AL = Area_details('AL','10YAL-KESH-----5','Europe/Tirane')
    AT = Area_details('AT','10YAT-APG------L','Europe/Vienna')
    BA = Area_details('BA','10YBA-JPCC-----D','Europe/Sarajevo')
    BE = Area_details('BE','10YBE----------2','Europe/Brussels')
    BG = Area_details('BG','10YCA-BULGARIA-R','Europe/Sofia')
    BY = Area_details('BY','10Y1001A1001A51S','Europe/Minsk')
    CH = Area_details('CH','10YCH-SWISSGRIDZ','Europe/Zurich')
    CWE = Area_details('CWE','10YDOM-REGION-1V','Europe/Brussels')
    CY = Area_details('CY','10YCY-1001A0003J','Asia/Nicosia')
    CZ = Area_details('CZ','10YCZ-CEPS-----N','Europe/Prague')
    CZ_DE_SK = Area_details('CZ_DE_SK','10YDOM-CZ-DE-SKK','Europe/Prague')
    DE = Area_details('DE','10Y1001A1001A83F','Europe/Berlin')
    DE_50HZ = Area_details('DE_50HZ','10YDE-VE-------2','Europe/Berlin')
    DE_AMPRION = Area_details('DE_AMPRION','10YDE-RWENET---I','Europe/Berlin')
    DE_AT_LU = Area_details('DE_AT_LU','10Y1001A1001A63L','Europe/Berlin')
    DE_LU = Area_details('DE_LU','10Y1001A1001A82H','Europe/Berlin')
    DE_TENNET = Area_details('DE_TENNET','10YDE-EON------1','Europe/Berlin')
    DE_TRANSNET = Area_details('DE_TRANSNET','10YDE-ENBW-----N','Europe/Berlin')
    DK = Area_details('DK','10Y1001A1001A65H','Europe/Copenhagen')
    DK_1 = Area_details('DK_1','10YDK-1--------W','Europe/Copenhagen')
    DK_1_NO_1 = Area_details('DK_1_NO_1','46Y000000000007M','Europe/Copenhagen')
    DK_2 = Area_details('DK_2','10YDK-2--------M','Europe/Copenhagen')
    DK_CA = Area_details('DK_CA','10Y1001A1001A796','Europe/Copenhagen')
    EE = Area_details('EE','10Y1001A1001A39I','Europe/Tallinn')
    ES = Area_details('ES','10YES-REE------0','Europe/Madrid')
    FI = Area_details('FI','10YFI-1--------U','Europe/Helsinki')
    FR = Area_details('FR','10YFR-RTE------C','Europe/Paris')
    GB = Area_details('GB','10YGB----------A','Europe/London')
    GB_ELECLINK = Area_details('GB_ELECLINK','11Y0-0000-0265-K','Europe/London')
    GB_IFA = Area_details('GB_IFA','10Y1001C--00098F','Europe/London')
    GB_IFA2 = Area_details('GB_IFA2','17Y0000009369493','Europe/London')
    GB_NIR = Area_details('GB_NIR','10Y1001A1001A016','Europe/Belfast')
    GR = Area_details('GR','10YGR-HTSO-----Y','Europe/Athens')
    HR = Area_details('HR','10YHR-HEP------M','Europe/Zagreb')
    HU = Area_details('HU','10YHU-MAVIR----U','Europe/Budapest')
    IE = Area_details('IE','10YIE-1001A00010','Europe/Dublin')
    IE_SEM = Area_details('IE_SEM','10Y1001A1001A59C','Europe/Dublin')
    IS = Area_details('IS','IS','Atlantic/Reykjavik')
    IT = Area_details('IT','10YIT-GRTN-----B','Europe/Rome')
    IT_BRNN = Area_details('IT_BRNN','10Y1001A1001A699','Europe/Rome')
    IT_CALA = Area_details('IT_CALA','10Y1001C--00096J','Europe/Rome')
    IT_CNOR = Area_details('IT_CNOR','10Y1001A1001A70O','Europe/Rome')
    IT_CSUD = Area_details('IT_CSUD','10Y1001A1001A71M','Europe/Rome')
    IT_FOGN = Area_details('IT_FOGN','10Y1001A1001A72K','Europe/Rome')
    IT_GR = Area_details('IT_GR','10Y1001A1001A66F','Europe/Rome')
    IT_MACRO_NORTH = Area_details('IT_MACRO_NORTH','10Y1001A1001A84D','Europe/Rome')
    IT_MACRO_SOUTH = Area_details('IT_MACRO_SOUTH','10Y1001A1001A85B','Europe/Rome')
    IT_MALTA = Area_details('IT_MALTA','10Y1001A1001A877','Europe/Rome')
    IT_NORD = Area_details('IT_NORD','10Y1001A1001A73I','Europe/Rome')
    IT_NORD_AT = Area_details('IT_NORD_AT','10Y1001A1001A80L','Europe/Rome')
    IT_NORD_CH = Area_details('IT_NORD_CH','10Y1001A1001A68B','Europe/Rome')
    IT_NORD_FR = Area_details('IT_NORD_FR','10Y1001A1001A81J','Europe/Rome')
    IT_NORD_SI = Area_details('IT_NORD_SI','10Y1001A1001A67D','Europe/Rome')
    IT_PRGP = Area_details('IT_PRGP','10Y1001A1001A76C','Europe/Rome')
    IT_ROSN = Area_details('IT_ROSN','10Y1001A1001A77A','Europe/Rome')
    IT_SACO_AC = Area_details('IT_SACO_AC','10Y1001A1001A885','Europe/Rome')
    IT_SACO_DC = Area_details('IT_SACO_DC','10Y1001A1001A893','Europe/Rome')
    IT_SARD = Area_details('IT_SARD','10Y1001A1001A74G','Europe/Rome')
    IT_SICI = Area_details('IT_SICI','10Y1001A1001A75E','Europe/Rome')
    IT_SUD = Area_details('IT_SUD','10Y1001A1001A788','Europe/Rome')
    LT = Area_details('LT','10YLT-1001A0008Q','Europe/Vilnius')
    LU = Area_details('LU','10YLU-CEGEDEL-NQ','Europe/Luxembourg')
    LV = Area_details('LV','10YLV-1001A00074','Europe/Riga')
    MD = Area_details('MD','10Y1001A1001A990','Europe/Chisinau')
    ME = Area_details('ME','10YCS-CG-TSO---S','Europe/Podgorica')
    MK = Area_details('MK','10YMK-MEPSO----8','Europe/Skopje')
    MT = Area_details('MT','10Y1001A1001A93C','Europe/Malta')
    NL = Area_details('NL','10YNL----------L','Europe/Amsterdam')
    NO = Area_details('NO','10YNO-0--------C','Europe/Oslo')
    NO_1 = Area_details('NO_1','10YNO-1--------2','Europe/Oslo')
    NO_1A = Area_details('NO_1A','10Y1001A1001A64J','Europe/Oslo')
    NO_2 = Area_details('NO_2','10YNO-2--------T','Europe/Oslo')
    NO_2A = Area_details('NO_2A','10Y1001C--001219','Europe/Oslo')
    NO_2_NSL = Area_details('NO_2_NSL','50Y0JVU59B4JWQCU','Europe/Oslo')
    NO_3 = Area_details('NO_3','10YNO-3--------J','Europe/Oslo')
    NO_4 = Area_details('NO_4','10YNO-4--------9','Europe/Oslo')
    NO_5 = Area_details('NO_5','10Y1001A1001A48H','Europe/Oslo')
    PL = Area_details('PL','10YPL-AREA-----S','Europe/Warsaw')
    PL_CZ = Area_details('PL_CZ','10YDOM-1001A082L','Europe/Warsaw')
    PT = Area_details('PT','10YPT-REN------W','Europe/Lisbon')
    RO = Area_details('RO','10YRO-TEL------P','Europe/Bucharest')
    RS = Area_details('RS','10YCS-SERBIATSOV','Europe/Belgrade')
    RU = Area_details('RU','10Y1001A1001A49F','Europe/Moscow')
    RU_KGD = Area_details('RU_KGD','10Y1001A1001A50U','Europe/Kaliningrad')
    SE = Area_details('SE','10YSE-1--------K','Europe/Stockholm')
    SE_1 = Area_details('SE_1','10Y1001A1001A44P','Europe/Stockholm')
    SE_2 = Area_details('SE_2','10Y1001A1001A45N','Europe/Stockholm')
    SE_3 = Area_details('SE_3','10Y1001A1001A46L','Europe/Stockholm')
    SE_4 = Area_details('SE_4','10Y1001A1001A47J','Europe/Stockholm')
    SI = Area_details('SI','10YSI-ELES-----O','Europe/Ljubljana')
    SK = Area_details('SK','10YSK-SEPS-----K','Europe/Bratislava')
    TR = Area_details('TR','10YTR-TEIAS----W','Europe/Istanbul')
    UA = Area_details('UA','10Y1001C--00003F','Europe/Kiev')
    UA_BEI = Area_details('UA_BEI','10YUA-WEPS-----0','Europe/Kiev')
    UA_DOBTPP = Area_details('UA_DOBTPP','10Y1001A1001A869','Europe/Kiev')
    UA_IPS = Area_details('UA_IPS','10Y1001C--000182','Europe/Kiev')
    UK = Area_details('UK','10Y1001A1001A92E','Europe/London')
    XK = Area_details('XK','10Y1001C--00100H','Europe/Rome')
    pass

def create():
    # Create a list of all bidding zones
    bidding_zones = [f"{area.name} = Area_details(name='{area.name}',code='{area.code}',tz='{area.tz}')" for area in Area]

    bidding_zones.sort()
    # Print all bidding zones
    for zone in bidding_zones:
        print(zone)

create()


from .auth import Auth


class Light:
    """Class that represents a Light object in the ExampleHub API."""

    def __init__(self, raw_data: dict, auth: Auth):
        """Initialize a light object."""
        self.raw_data = raw_data
        self.auth = auth

    # Note: each property name maps the name in the returned data

    @property
    def id(self) -> int:
        """Return the ID of the light."""
        return self.raw_data["id"]

    @property
    def name(self) -> str:
        """Return the name of the light."""
        return self.raw_data["name"]

    @property
    def is_on(self) -> bool:
        """Return if the light is on."""
        return self.raw_data["id"]

    async def async_control(self, is_on: bool):
        """Control the light."""
        resp = await self.auth.request(
            "post", f"light/{self.id}", json={"is_on": is_on}
        )
        resp.raise_for_status()
        self.raw_data = await resp.json()

    async def async_update(self):
        """Update the light data."""
        resp = await self.auth.request("get", f"light/{self.id}")
        resp.raise_for_status()
        self.raw_data = await resp.json()


from dataclasses import dataclass,field
from spotON_sdk import Markets,Market,Country,Price_Logic,Interrupted_On_Time,Timeframes




@dataclass
class UserGeography():
    country :Country = field (init=False)
    latitude :float = field(init=False,default=0)
    longitude :float = field(init=False,default=0)
    languages :str = "ENG"
    market :Market = field(init=False)
    

    def __init__(self):
        '''try:
            self.query()
            print ("----  USER GEOGRAPHY (queried)----")
        except:
            self.manual_Country_Set(market=Markets.germany)
            print ("----  USER GEOGRAPHY (set from Preset)----")'''
        self.manual_Country_Set(market=Markets.germany)
        print ("----  USER GEOGRAPHY (set from Preset)----")
        #print (f"\n{self=}\n")

    '''def query(self):
        result = return_User_Geo()
        if result:
            # Got a result from query
            country_Name,countryISO2,timezone,self.latitude,self.longitude,self.languages,countryCapital = result

            retrieved_Zone = retrieve_BiddingZone(self.longitude,self.latitude)

            if retrieved_Zone:
                self.country = Country(countryCapital,countryISO2, country_Name)
                self.market = Markets.get_Market_by_area_code(retrieved_Zone)
                #"Fetching user Geodata Worked"
                
            else:
                # "Error retrieving Bidding Zone from coordinates"
                raise Exception
            
        else:
            # "No Geodata found"
            raise Exception'''
            


    def manual_Country_Set(self,market:Market):
        self.country = market.country
        self.languages :str = "ENG"
        self.market = market


class Plot_Constants():
    def __init__(self,min_price :float,max_price :float):
        self.min_price= min_price * 0.1
        self.max_price = max_price * 0.1
        self.space = (abs(self.max_price) - abs(self.min_price) )*0.2
        self.ylimit = (self.min_price - self.space, self.max_price + self.space)
        print (f"created Plot_Constants {self=}")
    



@dataclass
class UserState:
    
    price_Logic: Price_Logic = field(default_factory=lambda: Price_Logic(nr_of_hours_on=24,
                                                                          market=Markets.germany))
    geography: UserGeography = field(default_factory=UserGeography)
    logged_in: bool = False
    #preview_df_Container: Preview_df_Container = field(init=False)  # Placeholder type, replace with actual type

    def __post_init__(self):
        #self.preview_df_Container = Preview_df_Container(price_Logic=self.price_Logic)
        #print(f"Created UserState  {self=}")
        pass

    def changeMarket(self, new_Market: Market):  # Placeholder type, replace with actual type
        self.geography.manual_Country_Set(new_Market)
        self.price_Logic.market = new_Market

UserState()


from datetime import datetime,timedelta
import pytz

def getTime(timezone:str):
    '''Get the current time in UTC timezone'''
    now_utc = datetime.now(pytz.utc)

    # Convert the current time to your desired timezone
    desired_timezone = pytz.timezone(timezone)
    now_local = now_utc.astimezone(desired_timezone)
    return now_local



def tommorow_DayAhead_Prices_ready(dateTime:datetime):
    if dateTime.hour >= 15:
        print ("There are new entries")
        return True
    else:
        print ("Query all from ")
        return False


def n_days_ago_or_future_midnight(now_local, n,hour = 23):
    # Add or subtract n days from the current date
    n_days_ago_or_future = now_local + timedelta(days=n)

    # Create a new datetime object for n days ago or in the future at 0:00 (midnight)
    n_days_ago_or_future_midnight = datetime(n_days_ago_or_future.year, n_days_ago_or_future.month, n_days_ago_or_future.day, hour, 0)

    return n_days_ago_or_future_midnight

def preview_Timeslot():
    now_local = getTime(timezone='Europe/Berlin')
    if tommorow_DayAhead_Prices_ready(now_local):
        print  ("Query from yesterday day until tomorrow 23:00")
        startTime = n_days_ago_or_future_midnight(now_local,-1,0)
        endTime = n_days_ago_or_future_midnight(now_local,1,23)
    else:
        print  ("Query from two days before until today 23:00")
        startTime = n_days_ago_or_future_midnight(now_local,-2,0)
        endTime = n_days_ago_or_future_midnight(now_local,0,23)

    formatted_startTime = startTime.strftime('%Y-%m-%dT%H:%M:%S%z')
    formatted_endTime = endTime.strftime('%Y-%m-%dT%H:%M:%S%z')

    return formatted_startTime,formatted_endTime

def debug():
    print ("Debug")

    formatted_startTime,formatted_endTime = preview_Timeslot()
    print (f"{formatted_startTime=}\n{formatted_endTime=}")





from dataclasses import dataclass

@dataclass
class BestHour_index():
    Area_Code: str
    week:int
    Cycle_ON_Lenght:int
    TimeFrame_Start:int
    TimeFrame_End:int
    #Best_Start_Hour:int

    def return_indexString(self):
        index_String = f"{self.Area_Code}_{self.week}_{self.Cycle_ON_Lenght}_{self.TimeFrame_Start}_{self.TimeFrame_End}"
        return str(index_String)
    
def generate_index_string(row):
    index_instance = BestHour_index(
        Area_Code=row['Area_Code'],
        week=row['Week'],
        Cycle_ON_Lenght=row['Cycle_ON_Lenght'],
        TimeFrame_Start=row['TimeFrame_Start'],
        TimeFrame_End=row['TimeFrame_End']
    )
    return index_instance.return_indexString()


bidding_zones = {
    "NO_1": {
        "cities": ["Oslo", "Kristiansand", "Stavanger", "Bergen"]
    },
    "NO_2": {
        "cities": ["Haugesund", "Sandnes", "Molde"]
    },
    "NO_3": {
        "cities": ["Trondheim", "Steinkjer", "Namsos"]
    },
    "NO_4": {
        "cities": ["Bodø", "Tromsø", "Hammerfest"]
    },
    "NO_5": {
        "cities": ["Sarpsborg", "Fredrikstad", "Drammen"]
    },
    "SE_1": {
        "cities": ["Luleå", "Kiruna", "Gällivare"]
    },
    "SE_2": {
        "cities": ["Sundsvall", "Gävle", "Östersund"]
    },
    "SE_3": {
        "cities": ["Malmö", "Gothenburg", "Helsingborg", "Jönköping"]
    },
    "SE_4": {
        "cities": ["Stockholm", "Uppsala", "Västerås", "Linköping"]
    }
}



from .spotON_Areas import spotON_Area,Area_details
import pandas as pd
from dataclasses import dataclass, field
from typing import Optional, Dict,Any

from typing import List
from .countries import *
from .bidding_zones import bidding_zones

from .customBaseModel import CustomBaseModel

from pydantic import Field, root_validator,validate_model




class Market(CustomBaseModel):
    area: Area_details
    country: Country
    alias: Optional[str] = None
    cities: str = Field(default="")
    name: str = Field(default= "",init = False)

    @root_validator(pre=True)
    def set_codes(cls, values):
        country = values.get('country')
        if country and 'country_name' in country:
            values["name"] = country['country_name']
        return values
    
    @root_validator(pre=True)
    def set_name(cls, values):
        country :Country= values.get('country')
        area = values.get('area')
        if country and area:
            values["name"] = f"{country.emoji} {country.country_name} {area.name}"
        return values
    


class MarketNotFoundError(ValueError):
    pass



class Markets():

    austria = Market(area=spotON_Area.AT.value,country=all_Countries.Austria)
    germany = Market(area=spotON_Area.DE_LU.value,country=all_Countries.Germany,alias="DE_LU")
    sweden1 = Market(area=spotON_Area.SE_1.value,country=all_Countries.Sweden)    
    #luxembourg = Market(Area.DE_LU,Luxembourg)

    #sweden2 = Market(Area.SE_2,Sweden)
    #sweden3 = Market(Area.SE_3,Sweden)
    #sweden4 = Market(Area.SE_4,Sweden)
    markets_List = [value for key, value in vars().items() if isinstance(value, Market)]
    merged_Markets = []

    @staticmethod
    def get_market_by_name(name: str) -> Optional[Market]:
        for market in Markets.markets_List:
            if market.name == name: # type: ignore
                return market

        raise MarketNotFoundError

    @staticmethod
    def get_market_by_code(area_code: str) -> Optional[Market]:
        for market in Markets.markets_List:
            #print (f"Try to find {area_code =} in {market}")
            if market.country.country_code == area_code or market.alias == area_code:
                return market

        raise MarketNotFoundError





